package jobqueue

import (
	"context"
	"fmt"
	"github.com/docker/go-connections/nat"
	"github.com/stretchr/testify/assert"
	"net"
	"os"
	"strconv"
	"testing"
	"time"
)

func startSFTPTestDockerImageAndContainer(t *testing.T, volume string, ctx context.Context) (string, error) {
	t.Helper()

	sftpPortMap := nat.PortMap{
		"22/tcp": []nat.PortBinding{
			{
				HostIP:   "0.0.0.0",
				HostPort: "",
			},
		},
	}

	sftpCmd := []string{"demo:secret:::upload"}
	sftpVolume := volume // Ihr Volume hier
	sftpPorts, err := startTestDockerImageAndContainer(t, ctx, "atmoz/sftp:alpine", sftpPortMap, sftpCmd, sftpVolume)
	if err != nil {
		return "", err
	}

	pp := *sftpPorts
	px, ok := pp["22/tcp"]
	if !ok {
		return "", fmt.Errorf("22/tcp port not found")
	}

	return px[0].HostPort, nil
	//
	//sshPorts, ok := sftpPorts["22/tcp"]
	//if !ok {
	//	return "", fmt.Errorf("22/tcp port not found")
	//}
	//
	//sshPort, ok := sshPorts[0].HostPort
	//if !ok {
	//	return "", fmt.Errorf("22/tcp port not found")
	//}
	//
	//// check if port is open
	//return sshPort, nil

	//t.Helper()
	//
	//cli, err := client.NewClientWithOpts(client.FromEnv, client.WithAPIVersionNegotiation())
	//if err != nil {
	//	return err
	//}
	//
	//imageName := "atmoz/sftp:alpine"
	//
	//reader, err := cli.ImagePull(ctx, imageName, types.ImagePullOptions{})
	//if err != nil {
	//	return err
	//}
	//
	////if debug image pull, comment out the following lines
	////_, _ = io.Copy(os.Stdout, reader)
	//_ = reader
	//
	//host, _, _ := net.SplitHostPort(cli.DaemonHost())
	//if host == "" || host == "unix" {
	//	host = DOCKER_TEST_HOST_IP
	//}
	//
	//hostConfig := &container.HostConfig{
	//	PortBindings: nat.PortMap{
	//		"22/tcp": []nat.PortBinding{
	//			{
	//				HostIP:   host,
	//				HostPort: port,
	//			},
	//		},
	//	},
	//}
	//
	//if volume != "" {
	//	hostConfig.Binds = append(hostConfig.Binds, volume+":/home/demo/upload")
	//}
	//
	//resp, err := cli.ContainerCreate(ctx, &container.Config{
	//	Image: imageName,
	//	Cmd:   []string{"demo:secret:::upload"},
	//}, hostConfig, nil, nil, "")
	//
	//if err != nil {
	//	return err
	//}
	//
	//if err := cli.ContainerStart(ctx, resp.ID, types.ContainerStartOptions{}); err != nil {
	//	return err
	//}
	//
	//go func() {
	//	<-ctx.Done()
	//
	//	timeout := 0
	//	stopOptions := container.StopOptions{
	//		Timeout: &timeout,
	//		Signal:  "SIGKILL",
	//	}
	//	newCtx, _ := context.WithTimeout(context.Background(), 60*time.Second)
	//	if err := cli.ContainerStop(newCtx, resp.ID, stopOptions); err != nil {
	//		t.Errorf("ContainerStop returned error: %v", err)
	//	}
	//	if err := cli.ContainerRemove(newCtx, resp.ID, types.ContainerRemoveOptions{
	//		Force: true,
	//	}); err != nil {
	//		t.Errorf("ContainerRemove returned error: %v", err)
	//	}
	//
	//}()
	//
	//statusCh, errCh := cli.ContainerWait(ctx, resp.ID, container.WaitConditionNotRunning)
	//select {
	//case err := <-errCh:
	//	if err != nil {
	//		// empty error means container exited normally (see container_wait.go)
	//		if err.Error() == "" {
	//			return nil
	//		}
	//
	//		return err
	//	}
	//case <-statusCh:
	//
	//}
	//
	//return nil
}

func TestSFTPCRunnerLocalToRemote(t *testing.T) {
	ctb := context.Background()
	ctx, cancel := context.WithCancel(ctb)
	t.Cleanup(func() {
		cancel()
		time.Sleep(1 * time.Second)
	})

	//listener, err := net.Listen("tcp", DOCKER_TEST_HOST_IP+":0")
	//if err != nil {
	//	t.Errorf("Unexpected error: %v", err)
	//	return
	//}
	//portAsInt := listener.Addr().(*net.TCPAddr).Port
	//portAsString := fmt.Sprintf("%d", portAsInt)
	//_ = listener.Close()

	var portAsString string
	var err error

	done := make(chan bool)
	go func() {
		portAsString, err = startSFTPTestDockerImageAndContainer(t, "", ctx)
		if err != nil {
			t.Errorf("Unexpected error: %v", err)
			cancel()
		}
		done <- true
	}()

	waitCtx, waitCancel := context.WithTimeout(ctx, 60*time.Second)
	defer waitCancel()
	for {
		conn, err := net.DialTimeout("tcp", net.JoinHostPort(DOCKER_TEST_HOST_IP, portAsString), 1*time.Second)
		if err == nil {
			err = conn.Close()
			assert.Nil(t, err)
			break
		}
		select {
		case <-waitCtx.Done():
			t.Error("Timeout waiting for container service")
			cancel()
			return
		default:
			time.Sleep(1 * time.Second)
		}
	}

	time.Sleep(1 * time.Second)

	tempDir := t.TempDir()
	// create 4 test files
	for i := 0; i < 4; i++ {
		_, err := os.Create(fmt.Sprintf("%s/testfile%d.txt", tempDir, i))
		if err != nil {
			t.Errorf("Unexpected error: %v", err)
			return
		}
	}

	portAsInt, err := strconv.Atoi(portAsString)

	sftpRunnable := &SFTPRunnable{
		Host:           DOCKER_TEST_HOST_IP,
		Port:           portAsInt,
		User:           "demo",
		Insecure:       true,
		Credential:     "secret",
		CredentialType: "password",

		SrcDir:            tempDir,
		DstDir:            "upload",
		TransferDirection: LocalToRemote,
	}

	result, err := sftpRunnable.Run()

	// Assertions
	assert.NoError(t, err)
	assert.Equal(t, ResultStatusSuccess, result.Status)
	assert.IsType(t, SFTPResult{}, result.Data)

	// check result.Data contains 4 files
	sftpResult := result.Data.FilesCopied
	assert.Equal(t, 4, len(sftpResult))

	cancel()

	select {
	case <-done:
		time.Sleep(1 * time.Second)
	case <-time.After(1 * time.Minute):
		t.Error("test hangs, timeout reached")
	}

}

func TestSFTPCRunnerRemoteToLocal(t *testing.T) {
	ctb := context.Background()
	ctx, cancel := context.WithCancel(ctb)
	t.Cleanup(func() {
		cancel()
		time.Sleep(1 * time.Second)
	})

	//listener, err := net.Listen("tcp", DOCKER_TEST_HOST_IP+":0")
	//if err != nil {
	//	t.Errorf("Unexpected error: %v", err)
	//	return
	//}
	//portAsInt := listener.Addr().(*net.TCPAddr).Port
	//portAsString := fmt.Sprintf("%d", portAsInt)
	//_ = listener.Close()

	tempSrcDir := t.TempDir()
	// create 4 test files
	for i := 0; i < 4; i++ {
		_, err := os.Create(fmt.Sprintf("%s/testfile%d.txt", tempSrcDir, i))
		if err != nil {
			t.Errorf("Unexpected error: %v", err)
			return
		}
	}

	var portAsString string
	var err error

	done := make(chan bool)
	go func() {
		portAsString, err = startSFTPTestDockerImageAndContainer(t, tempSrcDir, ctx)
		if err != nil {
			t.Errorf("Unexpected error: %v", err)
			cancel()
		}
		done <- true
	}()

	waitCtx, waitCancel := context.WithTimeout(ctx, 60*time.Second)
	defer waitCancel()
	for {
		conn, err := net.DialTimeout("tcp", net.JoinHostPort(DOCKER_TEST_HOST_IP, portAsString), 1*time.Second)
		if err == nil {
			err = conn.Close()
			assert.Nil(t, err)
			break
		}
		select {
		case <-waitCtx.Done():
			t.Error("Timeout waiting for container service")
			cancel()
			return
		default:
			time.Sleep(1 * time.Second)
		}
	}

	time.Sleep(1 * time.Second)

	tempDir := t.TempDir()

	portAsInt, err := strconv.Atoi(portAsString)
	assert.NoError(t, err)

	sftpRunnable := &SFTPRunnable{
		Host:              DOCKER_TEST_HOST_IP,
		Port:              portAsInt,
		User:              "demo",
		Insecure:          true,
		Credential:        "secret",
		CredentialType:    "password",
		SrcDir:            "upload", // Remote-Verzeichnis mit Dateien
		DstDir:            tempDir,
		TransferDirection: RemoteToLocal,
	}

	// Methode aufrufen
	result, err := sftpRunnable.Run()

	// Assertions
	assert.NoError(t, err)
	assert.Equal(t, ResultStatusSuccess, result.Status)
	assert.IsType(t, SFTPResult{}, result.Data)

	// check result.Data contains 4 files
	sftpResult := result.Data.FilesCopied
	assert.Equal(t, 4, len(sftpResult))

	// check files in tempDir
	files, err := os.ReadDir(tempDir)
	assert.NoError(t, err)
	assert.Equal(t, 4, len(files))

	cancel()

	select {
	case <-done:
		time.Sleep(1 * time.Second)
	case <-time.After(1 * time.Minute):
		t.Error("test hangs, timeout reached")
	}
}
