package jobqueue

import (
	"context"
	"github.com/docker/docker/api/types"
	"github.com/docker/docker/api/types/container"
	"github.com/docker/docker/client"
	"github.com/docker/go-connections/nat"
	"testing"
	"time"
)

const (
	DOCKER_TEST_HOST_IP = "172.17.0.1"
)

func startTestDockerImageAndContainer(t *testing.T, ctx context.Context, imageName string, portMap nat.PortMap, cmd []string, volume string) (*nat.PortMap, error) {
	t.Helper()

	cli, err := client.NewClientWithOpts(client.FromEnv, client.WithAPIVersionNegotiation())
	if err != nil {
		return nil, err
	}

	reader, err := cli.ImagePull(ctx, imageName, types.ImagePullOptions{})
	if err != nil {
		return nil, err
	}
	_ = reader

	//host, _, _ := net.SplitHostPort(cli.DaemonHost())
	//if host == "" || host == "unix" {
	//	host = DOCKER_TEST_HOST_IP
	//}

	hostConfig := &container.HostConfig{
		PortBindings: portMap,
	}

	if volume != "" {
		hostConfig.Binds = append(hostConfig.Binds, volume+":/home/demo/upload")
	}

	resp, err := cli.ContainerCreate(ctx, &container.Config{
		Image: imageName,
		Cmd:   cmd,
	}, hostConfig, nil, nil, "")

	if err != nil {
		return nil, err
	}

	if err := cli.ContainerStart(ctx, resp.ID, types.ContainerStartOptions{}); err != nil {
		return nil, err
	}

	go func() {
		<-ctx.Done()

		timeout := 0
		stopOptions := container.StopOptions{
			Timeout: &timeout,
		}
		newCtx, _ := context.WithTimeout(context.Background(), 60*time.Second)
		if err := cli.ContainerStop(newCtx, resp.ID, stopOptions); err != nil {
			t.Errorf("ContainerStop returned error: %v", err)
		}
		if err := cli.ContainerRemove(newCtx, resp.ID, types.ContainerRemoveOptions{
			Force: true,
		}); err != nil {
			t.Errorf("ContainerRemove returned error: %v", err)
		}
	}()

	inspected, err := cli.ContainerInspect(ctx, resp.ID)
	if err != nil {
		return nil, err
	}

	return &inspected.NetworkSettings.Ports, nil
}
